`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineer: Ulrich Zoltn
// 
// Create Date:    09/18/2006
// Module Name:    mouse_displayer
// Project Name: 	 PmodPS2_Demo
// Target Devices: Nexys3
// Tool versions:  ISE 14.2
// Description:  Mouse position is received from the mouse_controller, horizontal and
// 				  vertical counters are received from vga_module and if the counters
// 				  are inside the mouse cursor bounds, then the mouse is sent to the
// 				  screen instead of the received pixels. The mouse cursor is 16x16
// 				  pixels and used 2 colors: white and black. For the color encoding
// 				  2 bits are used to be able to use transparency. The cursor is stored
// 				  in a distributed ram memory 256x2bits in dimension. If the current
// 				  pixel of the mouse is "00" then output color is black, if "01" then
// 				  white and if "10" or "11" then the pixel is perfectly transparent
// 				  and the input colors are output. This way, the mouse will not be a
// 				  16x16 square and will have an arrow shape.
// 				  Memory address is composed from the difference of the vga counters
// 				  and mouse position. xdiff is the difference on 4 bits (because cursor
// 				  is 16 pixels width) between the horizontal vga counter and the xpos
// 				  of the mouse. ydiff is the difference on 4 bits (because cursor
// 				  has 16 pixels in height) between the vertical vga counter and the
// 				  ypos of the mouse. By concatenating ydiff and xidff (in this order)
// 				  the memory address of the current pixel is obtained.
// 				  Distributed memory is forced by the attributes, because the block
// 				  rams are entirely used for storing images.
// 				  If blank input from the vga_module is active, this means that current
// 				  pixel is not inside visible screen and color outputs are set to black
//
//				   ----------------------------------------------------------------------
//  													Port definitions
//				   ----------------------------------------------------------------------
// 				  clk            - global clock signal (100MHz)
// 				  pixel_clk      - input pin, from pixel_clock_switcher, the clock used
// 				                 - by the vga_controller for the currently used
// 				                 - resolution, generated by a dcm. 25MHz for 640x480
// 				                 - and 40MHz for 800x600. This clock is used to read
// 				                 - pixels from memory and output data on color outputs.
// 				  xpos           - input pin, 10 bits, from mouse_controller
// 				                 - the x position of the mouse relative to the upper
// 				                 - left corner
// 				  ypos           - input pin, 10 bits, from mouse_controller
// 				                 - the y position of the mouse relative to the upper
// 				                 - left corner
// 				  hcount         - input pin, 11 bits, from vga_module
// 				                 - the horizontal counter from the vga_controller
// 				                 - tells the horizontal position of the current pixel
// 				                 - on the screen from left to right.
// 				  vcount         - input pin, 11 bits, from vga_module
// 				                 - the vertical counter from the vga_controller
// 				                 - tells the vertical position of the currentl pixel
// 				                 - on the screen from top to bottom.
// 				  blank          - input pin, from vga_module
// 				                 - if active, current pixel is not in visible area,
// 				                 - and color outputs should be set on 0.
// 				  red_in         - input pin, 3 bits, from effects_layer
// 				                 - red channel input of the image to be displayed
// 				  green_in       - input pin, 3 bits, from effects_layer
// 				                 - green channel input of the image to be displayed
// 				  blue_in        - input pin, 2 bits, from effects_layer
// 				                 - blue channel input of the image to be displayed
// 				  red_out        - output pin, 3 bits, to vga hardware module.
// 				                 - red output channel
// 				  green_out      - output pin, 3 bits, to vga hardware module.
// 				                 - green output channel
// 				  blue_out       - output pin, 2 bits, to vga hardware module.
// 				                 - blue output channel
//
// Revision History: 
//							Revision 0.00 - File created (UlrichZ)
//							Revision 1.00 - Added Comments and Converted to Verilog (Josh Sackos)
//////////////////////////////////////////////////////////////////////////////////////////

// =======================================================================================
// 										  Define Module
// =======================================================================================
module mouse_displayer(
    clk,
    pixel_clk,
    xpos,
    ypos,
    hcount,
    vcount,
	 blank,
    red_in,
    green_in,
    blue_in,
    red_out,
    green_out,
    blue_out
);

// =======================================================================================
// 										Port Declarations
// =======================================================================================

			input clk;
			input pixel_clk;
			input [9:0] xpos;
			input [9:0] ypos;
			input [10:0] hcount;
			input [10:0] vcount;
			input blank;
			input [2:0] red_in;
			input [2:0] green_in;
			input [1:0] blue_in;
			output [2:0] red_out;
			output [2:0] green_out;
			output [1:0] blue_out;

// =======================================================================================
// 							  Parameters, Registers, and Wires
// =======================================================================================

			reg [2:0] red_out;
			reg [2:0] green_out;
			reg [1:0] blue_out;

			// the memory that holds the cursor.
			// 00 - black
			// 01 - white
			// 1x - transparent
			parameter [1:0] mouserom[0:255] = {
			2'b00, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b00, 2'b00, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b01, 2'b01, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b01, 2'b00, 2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b00, 2'b11, 2'b11, 2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b00, 2'b11, 2'b11, 2'b11, 2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b00, 2'b01, 2'b01, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11,
			2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b00, 2'b00, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11, 2'b11
			};

			// width and height of cursor.
			parameter [4:0] OFFSET = 5'b10000;		// 16

			//signal rompos: std_logic_vector(7 downto 0) $ (others => '0');

			// pixel from the display memory, representing currently displayed
			// pixel of the cursor, if the cursor is being display at this point
			reg [1:0]      mousepixel;
			// when high, enables displaying of the cursor, and reading the
			// cursor memory.
			reg             enable_mouse_display;

			// difference in range 0-15 between the vga counters and mouse position
			reg [3:0]       xdiff = 4'd0;
			reg [3:0]       ydiff = 4'd0;

// =======================================================================================
// 							  		   Implementation
// =======================================================================================
   
			// compute xdiff
			always @(posedge clk)
			begin: x_diff
			reg [10:0] x_temp_diff;
			begin
				x_temp_diff = hcount - ({1'b0, xpos});
				xdiff <= x_temp_diff[3:0];
			end
			end

			// compute ydiff   
			always @(posedge clk) begin: y_diff
			reg [10:0] y_temp_diff;
			begin
				y_temp_diff = vcount - ({1'b0, ypos});
				ydiff <= y_temp_diff[3:0];
			end
			end

			//   rompos <= (ydiff & xdiff) when rising_edge(clk);

			// read pixel from memory at address obtained by concatenation of
			// ydiff and xdiff
			always @(posedge pixel_clk) begin
				mousepixel <= mouserom[{ydiff, xdiff}];
			end
			// set enable_mouse_display high if vga counters inside cursor block

			always @(posedge pixel_clk) begin: enable_mouse
				if (hcount >= xpos & hcount < (xpos + OFFSET) & vcount >= ypos & vcount < (ypos + OFFSET)) begin
					enable_mouse_display <= 1'b1;
				end
				else begin
					enable_mouse_display <= 1'b0;
				end
			end

			// if cursor display is enabled, then, according to pixel
			// value, set the output color channels.
			always @(posedge pixel_clk) begin
				// if in visible screen
				if (blank == 1'b0) begin
					// in display is enabled
					if (enable_mouse_display == 1'b1) begin
						// white pixel of cursor
						if (mousepixel == 2'b01) begin
							red_out <= {3{1'b1}};
							green_out <= {3{1'b1}};
							blue_out <= {2{1'b1}};
						end
						// black pixel of cursor
						else if (mousepixel == 2'b00) begin
							red_out <= {3{1'b0}};
							green_out <= {3{1'b0}};
							blue_out <= {2{1'b0}};
						end
						else begin
							// transparent pixel of cursor
							// let input pass to output
							red_out <= red_in;
							green_out <= green_in;
							blue_out <= blue_in;
						end
					end
					else begin
						// cursor display is not enabled
						// let input pass to output.
						red_out <= red_in;
						green_out <= green_in;
						blue_out <= blue_in;
					end
				end
				else begin
					// not in visible screen, black outputs.
					red_out <= {3{1'b0}};
					green_out <= {3{1'b0}};
					blue_out <= {2{1'b0}};
				end
			end
   
endmodule
