`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineer: Ulrich Zoltn
// 
// Create Date:    09/18/2006
// Module Name:    vga_controller_640_60
// Project Name: 	 PmodPS2_Demo
// Target Devices: Nexys3
// Tool versions:  ISE 14.2
// Description:  Please read the following article on the web regarding the
// 				  vga video timings:
// 				  http://www.epanorama.net/documents/pc/vga_timing.html
//
// 				  This module generates the video synch pulses for the monitor to
// 				  enter 640x480@60Hz resolution state. It also provides horizontal
// 				  and vertical counters for the currently displayed pixel and a blank
// 				  signal that is active when the pixel is not inside the visible screen
// 				  and the color outputs should be reset to 0.
//
// 				  timing diagram for the horizontal synch signal (HS)
// 				  0                         648    744           800 (pixels)
// 				  -------------------------|______|-----------------
// 				  timing diagram for the vertical synch signal (VS)
// 				  0                                  482    484  525 (lines)
// 				  -----------------------------------|______|-------
//
// 				  The blank signal is delayed one pixel clock period (40ns) from where
// 				  the pixel leaves the visible screen, according to the counters, to
// 				  account for the pixel pipeline delay. This delay happens because
// 				  it takes time from when the counters indicate current pixel should
// 				  be displayed to when the color data actually arrives at the monitor
// 				  pins (memory read delays, synchronization delays).
//
//				   ----------------------------------------------------------------------
//  				   								Port definitions
//				   ----------------------------------------------------------------------
// 				  rst               - global reset signal
// 				  pixel_clk         - input pin, from dcm_25MHz
// 				                    - the clock signal generated by a DCM that has
// 				                    - a frequency of 25MHz.
// 				  HS                - output pin, to monitor
// 				                    - horizontal synch pulse
// 				  VS                - output pin, to monitor
// 				                    - vertical synch pulse
// 				  hcount            - output pin, 11 bits, to clients
// 				                    - horizontal count of the currently displayed
// 				                    - pixel (even if not in visible area)
// 				  vcount            - output pin, 11 bits, to clients
// 				                    - vertical count of the currently active video
// 				                    - line (even if not in visible area)
// 				  blank             - output pin, to clients
// 				                    - active when pixel is not in visible area.
//
// Revision History: 
//							Revision 0.00 - File created (UlrichZ)
//							Revision 1.00 - Added Comments and Converted to Verilog (Josh Sackos)
//////////////////////////////////////////////////////////////////////////////////////////

// =======================================================================================
// 										  Define Module
// =======================================================================================
module vga_controller_640_60(
    rst,
    pixel_clk,
    HS,
    VS,
    hcount,
    vcount,
    blank
);

// =======================================================================================
// 										Port Declarations
// =======================================================================================

			input rst;
			input pixel_clk;
			output HS;
			output VS;
			output [10:0] hcount;
			output [10:0] vcount;
			output blank;

// =======================================================================================
// 							  Parameters, Registers, and Wires
// =======================================================================================

			// Output wire and reg
			reg HS;
			reg VS;
			wire [10:0] hcount;
			wire [10:0] vcount;
			reg blank;

			// maximum value for the horizontal pixel counter
			parameter [10:0] HMAX = 11'b01100100000; // 800
			// maximum value for the vertical pixel counter
			parameter [10:0] VMAX = 11'b01000001101; // 525
			// total number of visible columns
			parameter [10:0] HLINES = 11'b01010000000; // 640
			// value for the horizontal counter where front porch ends
			parameter [10:0] HFP = 11'b01010001000; // 648
			// value for the horizontal counter where the synch pulse ends
			parameter [10:0] HSP = 11'b01011101000; // 744
			// total number of visible lines
			parameter [10:0] VLINES = 11'b00111100000; // 480
			// value for the vertical counter where the front porch ends
			parameter [10:0] VFP = 11'b00111100010; // 482
			// value for the vertical counter where the synch pulse ends
			parameter [10:0] VSP = 11'b00111100100; // 484
			// polarity of the horizontal and vertical synch pulse
			// only one polarity used, because for this resolution they coincide.
			parameter SPP = 1'b0;


			// horizontal and vertical counters
			reg [10:0] hcounter;
			reg [10:0] vcounter;

			// active when inside visible screen area.
			wire video_enable;

// =======================================================================================
// 							  		   Implementation
// =======================================================================================

			// output horizontal and vertical counters
			assign hcount = hcounter;
			assign vcount = vcounter;

			// blank is active when outside screen visible area
			// color output should be blacked (put on 0) when blank in active
			// blank is delayed one pixel clock period from the video_enable
			// signal to account for the pixel pipeline delay.
			always @(posedge pixel_clk) begin
					blank <= ~video_enable;
			end

			// increment horizontal counter at pixel_clk rate
			// until HMAX is reached, then reset and keep counting
			always @(posedge pixel_clk or posedge rst) begin
					if(rst == 1'b1) begin
						hcounter <= 11'd0;
					end
					else if(hcounter == HMAX) begin
						hcounter <= 11'd0;
					end
					else begin
						hcounter <= hcounter + 1'b1;
					end
			end

			// increment vertical counter when one line is finished
			// (horizontal counter reached HMAX)
			// until VMAX is reached, then reset and keep counting
			always @(posedge pixel_clk or posedge rst) begin
					if(rst == 1'b1) begin
						vcounter <= 11'd0;
					end
					else if(hcounter == HMAX) begin
						if(vcounter == VMAX) begin
							vcounter <= 11'd0;
						end
						else begin
							vcounter <= vcounter + 1'b1;
						end
					end
			end

			// generate horizontal synch pulse
			// when horizontal counter is between where the
			// front porch ends and the synch pulse ends.
			// The HS is active (with polarity SPP) for a total of 96 pixels.
			always @(posedge pixel_clk) begin
					if(hcounter >= HFP && hcounter < HSP) begin
						HS <= SPP;
					end
					else begin
						HS <= ~SPP;
					end
			end

			// generate vertical synch pulse
			// when vertical counter is between where the
			// front porch ends and the synch pulse ends.
			// The VS is active (with polarity SPP) for a total of 2 video lines
			// = 2*HMAX = 1600 pixels.
			always @(posedge pixel_clk) begin
					if(vcounter >= VFP && vcounter < VSP) begin
						VS <= SPP;
					end
					else begin
						VS <= ~SPP;
					end
			end

			// enable video output when pixel is in visible area
			assign video_enable = (hcounter < HLINES && vcounter < VLINES) ? 1'b1 : 1'b0;

endmodule
