`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////////////
// Company: Digilent Inc.
// Engineer: Ulrich Zoltn
// 
// Create Date:    09/18/2006
// Module Name:    vga_selector
// Project Name: 	 PmodPS2_Demo
// Target Devices: Nexys3
// Tool versions:  ISE 14.2
// Description:  Depending on the input signal resolution, the output signals from
// 				  the vga controllers are selected and sent to other logic in the
// 				  design that needs these outputs, or to the monitor.
//
//				   ----------------------------------------------------------------------
//  				  									Port definitions
//				   ----------------------------------------------------------------------
// 				  resolution        - input pin, from resolution_swticher
//                   - selects active resolution
// 				  HS_640_60         - input pin, from vga_controller_640_60
// 				                    - horizontal synch signal for 640x480
// 				  VS_640_60         - input pin, from vga_controller_640_60
// 				                    - vertical synch signal for 640x480
// 				  HS_800_60         - input pin, from vga_controller_800_60
// 				                    - horizontal synch signal for 800x600
// 				  VS_800_60         - input pin, from vga_controller_800_60
//                   - vertical synch signal for 800x600
// 				  blank_640_60      - input pin, from vga_controller_640_60
// 				                    - blank signal for 640x480
// 				  blank_800_60      - input pin, from vga_controller_800_60
// 				                    - blank signal for 800x600
// 				  hcount_640_60     - input pin, 11 bits, from vga_controller_640_60
// 				                    - horizontal pixel counter for 640x480
// 				  hcount_800_60     - input pin, 11 bits, from vga_controller_800_60
// 				                    - horizontal pixel counter for 800x600
// 				  vcount_640_60     - input pin, 11 bits, from vga_controller_640_60
// 				                    - vertical lines counter for 640x480
// 				  vcount_800_60     - input pin, 11 bits, from vga_controller_800_60
// 				                    - vertical lines counter for 800x600
// 				  hs                - output pin, to monitor
// 				                    - selected horizontal synch signal
// 				  vs                - output pin, to monitor
// 				                    - selected vertical synch signal
// 				  blank             - output pin, to clients
// 				                    - selected blank signal
// 				  hcount            - output pin, 11 bits, to clients
// 				                    - selected horizontal pixel counter
// 				  vcount            - output pin, 11 bits, to clients
// 				                    - selected vertical lines counter
//
// Revision History: 
//							Revision 0.00 - File created (UlrichZ)
//							Revision 1.00 - Added Comments and Converted to Verilog (Josh Sackos)
//////////////////////////////////////////////////////////////////////////////////////////

// =======================================================================================
// 										  Define Module
// =======================================================================================
module vga_selector(
    resolution,
    HS_640_60,
    VS_640_60,
    HS_800_60,
    VS_800_60,
    blank_640_60,
    blank_800_60,
    hcount_640_60,
    hcount_800_60,
    vcount_640_60,
    vcount_800_60,
    hs,
    vs,
    blank,
    hcount,
    vcount
    );

// =======================================================================================
// 										Port Declarations
// =======================================================================================

			input resolution;
			input HS_640_60;
			input VS_640_60;
			input HS_800_60;
			input VS_800_60;
			input blank_640_60;
			input blank_800_60;
			input [10:0] hcount_640_60;
			input [10:0] hcount_800_60;
			input [10:0] vcount_640_60;
			input [10:0] vcount_800_60;
			output hs;
			output vs;
			output blank;
			output [10:0] hcount;
			output [10:0] vcount;

// =======================================================================================
// 							  Parameters, Registers, and Wires
// =======================================================================================

			wire [10:0] hcount, vcount;
			wire hs, vs, blank;

// =======================================================================================
// 							  		   Implementation
// =======================================================================================

			// select horizontal synch pulse signal depending on the
			// resolution used
			assign hs = (resolution == 1'b1) ? HS_800_60 : HS_640_60;

			// select vertical synch pulse signal depending on the
			// resolution used
			assign vs = (resolution == 1'b1) ? VS_800_60 : VS_640_60;

			// select blank signal depending on the resolution used
			assign blank = (resolution == 1'b1) ? blank_800_60 : blank_640_60;

			// select horizontal counter depending on the resolution used
			assign hcount = (resolution == 1'b1) ? hcount_800_60 : hcount_640_60;

			// select vertical counter depending on the resolution used
			assign vcount = (resolution == 1'b1) ? vcount_800_60 : vcount_640_60;


endmodule
